@@ -1,6 +1,7 @@
 #!/usr/bin/env python3
 
-# pylint: disable=C0111
+# pylint: disable=C0111, redefined-outer-name
+
 
 from collections import defaultdict, OrderedDict
 import os
@@ -8,16 +9,12 @@ import time
 import tempfile
 import copy
 
-
-import pytest
 import numpy as np
 import xarray as xr
 import pandas as pd
+import pytest
 
-from pyndl import ndl, count
-
-slow = pytest.mark.skipif(not pytest.config.getoption("--runslow"),
-                          reason="need --runslow option to run")
+from pyndl import ndl, count, io
 
 TEST_ROOT = os.path.join(os.path.pardir, os.path.dirname(__file__))
 FILE_PATH_SIMPLE = os.path.join(TEST_ROOT, "resources/event_file_simple.tab.gz")
@@ -52,7 +49,7 @@ def result_dict_ndl():
 
 @pytest.fixture(scope='module')
 def result_dict_ndl_generator():
-    return ndl.dict_ndl(ndl.events_from_file(FILE_PATH_SIMPLE), ALPHA, BETAS)
+    return ndl.dict_ndl(io.events_from_file(FILE_PATH_SIMPLE), ALPHA, BETAS)
 
 
 @pytest.fixture(scope='module')
@@ -66,8 +63,8 @@ def result_continue_learning():
     part_1 = events_simple.head(CONTINUE_SPLIT_POINT)
     part_2 = events_simple.tail(len(events_simple) - CONTINUE_SPLIT_POINT)
 
-    assert len(part_1) > 0
-    assert len(part_2) > 0
+    assert len(part_1) > 0  # pylint: disable=len-as-condition
+    assert len(part_2) > 0  # pylint: disable=len-as-condition
 
     part_path_1 = os.path.join(TMP_PATH, "event_file_simple_1.tab.gz")
     part_path_2 = os.path.join(TMP_PATH, "event_file_simple_2.tab.gz")
@@ -81,11 +78,8 @@ def result_continue_learning():
 
     del events_simple, part_1, part_2
 
-    result_part = ndl.ndl(part_path_1,
-                          ALPHA, BETAS)
-
-    result = ndl.ndl(part_path_2, ALPHA, BETAS,
-                     weights=result_part)
+    result_part = ndl.ndl(part_path_1, ALPHA, BETAS)
+    result = ndl.ndl(part_path_2, ALPHA, BETAS, weights=result_part)
 
     return result
 
@@ -109,8 +103,8 @@ def test_exceptions():
 
     with pytest.raises(ValueError) as e_info:
         ndl.ndl(FILE_PATH_SIMPLE, ALPHA, BETAS, method='threading',
-                len_sublists=-1)
-        assert e_info == "'len_sublists' must be larger then one"
+                n_outcomes_per_job=-1)
+        assert e_info == "'n_outcomes_per_job' must be larger then one"
 
     with pytest.raises(ValueError) as e_info:
         ndl.dict_ndl(FILE_PATH_SIMPLE, ALPHA, BETAS, make_data_array="magic")
@@ -121,17 +115,53 @@ def test_exceptions():
         assert e_info == "remove_duplicates must be None, True or False"
 
     with pytest.raises(ValueError) as e_info:
-        ndl.ndl(FILE_PATH_SIMPLE, ALPHA, BETAS, remove_duplicates="magic")
+        ndl.ndl(FILE_PATH_SIMPLE, ALPHA, BETAS, method='threading', remove_duplicates="magic")
         assert e_info == "remove_duplicates must be None, True or False"
 
+    with pytest.raises(FileNotFoundError, match="No such file or directory") as e_info:
+        ndl.ndl(FILE_PATH_SIMPLE, ALPHA, BETAS, method='threading', temporary_directory="./magic")
+
+    with pytest.raises(ValueError, match="events_per_file has to be larger than 1") as e_info:
+        ndl.ndl(FILE_PATH_SIMPLE, ALPHA, BETAS, method='threading', events_per_temporary_file=1)
+
+    with pytest.raises(AttributeError, match="weights does not have attributes "
+                       "and no attrs argument is given.") as e_info:
+        ndl.data_array(dict())
+
+#    # Test usually exeeds memory limit; It demands ~32GB of RAM.
+#    with pytest.raises(ValueError, match="Neither number of cues nor outcomes "
+#                       "shall exceed 4294967295 for now. See "
+#                       "https://github.com/quantling/pyndl/issues/169") as e_info:
+#        ndl.ndl(FILE_PATH_SIMPLE, ALPHA, BETAS,
+#                weights=xr.DataArray(np.zeros(shape=(4294967295 + 1, 1))))
+
+
+def test_generator_learning():
+    events = io.events_from_file(FILE_PATH_SIMPLE)
+    result_ndl_gen = ndl.ndl(events, ALPHA, BETAS, method='threading')
+    result_ndl = ndl.ndl(FILE_PATH_SIMPLE, ALPHA, BETAS, method='threading')
+
+    unequal, unequal_ratio = compare_arrays(FILE_PATH_SIMPLE,
+                                            result_ndl_gen,
+                                            result_ndl)
+    print(result_ndl_gen)
+    print('%.2f ratio unequal' % unequal_ratio)
+    assert len(unequal) == 0  # pylint: disable=len-as-condition
+
+
+def test_data_array_cast():
+    result_ndl = ndl.ndl(FILE_PATH_SIMPLE, ALPHA, BETAS, method='threading')
+    casted_result = ndl.data_array(result_ndl)
+    assert isinstance(casted_result, xr.DataArray) and (result_ndl == casted_result).all()
+
 
 def test_continue_learning_dict():
     events_simple = pd.read_csv(FILE_PATH_SIMPLE, sep="\t")
     part_1 = events_simple.head(CONTINUE_SPLIT_POINT)
     part_2 = events_simple.tail(len(events_simple) - CONTINUE_SPLIT_POINT)
 
-    assert len(part_1) > 0
-    assert len(part_2) > 0
+    assert len(part_1) > 0  # pylint: disable=len-as-condition
+    assert len(part_2) > 0  # pylint: disable=len-as-condition
 
     part_path_1 = os.path.join(TMP_PATH, "event_file_simple_1.tab.gz")
     part_path_2 = os.path.join(TMP_PATH, "event_file_simple_2.tab.gz")
@@ -174,9 +204,10 @@ def test_continue_learning_dict_ndl_data_array(result_dict_ndl, result_dict_ndl_
                                             continue_from_data_array)
     print(continue_from_data_array)
     print('%.2f ratio unequal' % unequal_ratio)
-    assert len(unequal) == 0
+    assert len(unequal) == 0  # pylint: disable=len-as-condition
 
 
+@pytest.mark.nolinux
 def test_continue_learning(result_continue_learning, result_ndl_openmp):
     assert result_continue_learning.shape == result_ndl_openmp.shape
 
@@ -188,9 +219,10 @@ def test_continue_learning(result_continue_learning, result_ndl_openmp):
                                             result_continue_learning,
                                             result_ndl_openmp)
     print('%.2f ratio unequal' % unequal_ratio)
-    assert len(unequal) == 0
+    assert len(unequal) == 0  # pylint: disable=len-as-condition
 
 
+@pytest.mark.nolinux
 def test_save_to_netcdf4(result_ndl_openmp):
     weights = result_ndl_openmp.copy()  # avoids changing shared test data
     path = os.path.join(TMP_PATH, "weights.nc")
@@ -207,6 +239,7 @@ def test_save_to_netcdf4(result_ndl_openmp):
         assert value == weights_continued_read.attrs[key]
 
 
+@pytest.mark.nolinux
 def test_return_values(result_dict_ndl, result_dict_ndl_data_array, result_ndl_threading, result_ndl_openmp):
     # dict_ndl
     assert isinstance(result_dict_ndl, defaultdict)
@@ -217,20 +250,26 @@ def test_return_values(result_dict_ndl, result_dict_ndl_data_array, result_ndl_t
     assert isinstance(result_ndl_threading, xr.DataArray)
 
 
+@pytest.mark.nolinux
+def test_provide_temporary_directory():
+    with tempfile.TemporaryDirectory(dir=TMP_PATH) as temporary_directory:
+        ndl.ndl(FILE_PATH_SIMPLE, ALPHA, BETAS, temporary_directory=temporary_directory)
+
+
 # Test internal consistency
 
 def test_dict_ndl_vs_ndl_threading(result_dict_ndl, result_ndl_threading):
     unequal, unequal_ratio = compare_arrays(FILE_PATH_SIMPLE, result_dict_ndl,
                                             result_ndl_threading)
     print('%.2f ratio unequal' % unequal_ratio)
-    assert len(unequal) == 0
+    assert len(unequal) == 0  # pylint: disable=len-as-condition
 
 
 def test_dict_ndl_vs_dict_ndl_generator(result_dict_ndl, result_dict_ndl_generator):
     unequal, unequal_ratio = compare_arrays(FILE_PATH_SIMPLE, result_dict_ndl,
                                             result_dict_ndl_generator)
     print('%.2f ratio unequal' % unequal_ratio)
-    assert len(unequal) == 0
+    assert len(unequal) == 0  # pylint: disable=len-as-condition
 
 
 def test_dict_ndl_data_array_vs_ndl_threading(result_ndl_threading):
@@ -239,6 +278,16 @@ def test_dict_ndl_data_array_vs_ndl_threading(result_ndl_threading):
     unequal, unequal_ratio = compare_arrays(FILE_PATH_SIMPLE, result_dict_ndl,
                                             result_ndl_threading)
     print('%.2f ratio unequal' % unequal_ratio)
+    assert len(unequal) == 0  # pylint: disable=len-as-condition
+
+
+def test_ordering_of_temporary_event_files(result_dict_ndl):
+    result_ndl = ndl.ndl(FILE_PATH_SIMPLE, ALPHA, BETAS, method='threading',
+                         events_per_temporary_file=2)
+
+    unequal, unequal_ratio = compare_arrays(FILE_PATH_SIMPLE, result_dict_ndl,
+                                            result_ndl)
+    print('%.2f ratio unequal' % unequal_ratio)
     assert len(unequal) == 0
 
 
@@ -249,17 +298,19 @@ def test_multiple_cues_dict_ndl_vs_ndl_threading():
     unequal, unequal_ratio = compare_arrays(FILE_PATH_MULTIPLE_CUES, result_dict_ndl,
                                             result_ndl_threading)
     print('%.2f ratio unequal' % unequal_ratio)
-    assert len(unequal) == 0
+    assert len(unequal) == 0  # pylint: disable=len-as-condition
 
 
+@pytest.mark.nolinux
 def test_dict_ndl_vs_ndl_openmp(result_dict_ndl, result_ndl_openmp):
     result_dict_ndl = ndl.dict_ndl(FILE_PATH_SIMPLE, ALPHA, BETAS)
     unequal, unequal_ratio = compare_arrays(FILE_PATH_SIMPLE, result_dict_ndl,
                                             result_ndl_openmp)
     print('%.2f ratio unequal' % unequal_ratio)
-    assert len(unequal) == 0
+    assert len(unequal) == 0  # pylint: disable=len-as-condition
 
 
+@pytest.mark.nolinux
 def test_meta_data(result_dict_ndl, result_dict_ndl_data_array, result_ndl_openmp, result_ndl_threading):
     attributes = {'cython', 'cpu_time', 'hostname', 'xarray', 'wall_time',
                   'event_path', 'number_events', 'username', 'method', 'date', 'numpy',
@@ -306,7 +357,7 @@ def test_compare_weights_ndl2(result_dict_ndl):
     unequal, unequal_ratio = compare_arrays(FILE_PATH_SIMPLE, result_ndl2, result_dict_ndl)
     print(set(outcome for outcome, *_ in unequal))
     print('%.2f ratio unequal' % unequal_ratio)
-    assert len(unequal) == 0
+    assert len(unequal) == 0  # pylint: disable=len-as-condition
 
 
 def test_multiple_cues_dict_ndl_vs_ndl2():
@@ -341,7 +392,7 @@ def test_multiple_cues_dict_ndl_vs_ndl2():
     unequal, unequal_ratio = compare_arrays(FILE_PATH_MULTIPLE_CUES, result_ndl2, result_python)
     print(set(outcome for outcome, *_ in unequal))
     print('%.2f ratio unequal' % unequal_ratio)
-    assert len(unequal) == 0
+    assert len(unequal) == 0  # pylint: disable=len-as-condition
 
 
 def test_compare_weights_rescorla_vs_ndl2():
@@ -384,43 +435,42 @@ def test_compare_weights_rescorla_vs_ndl2():
 
     unequal, unequal_ratio = compare_arrays(FILE_PATH_SIMPLE, result_ndl2, result_rescorla)
     print('%.2f ratio unequal' % unequal_ratio)
-    assert len(unequal) == 0
+    assert len(unequal) == 0  # pylint: disable=len-as-condition
 
 
-@slow
+@pytest.mark.runslow
 def test_compare_time_dict_inplace_parallel_thread():
     file_path = os.path.join(TEST_ROOT, 'resources/event_file_many_cues.tab.gz')
-    cue_map, outcome_map, all_outcomes = generate_mapping(file_path)
 
     result_dict_ndl, duration_not_parallel = clock(ndl.dict_ndl, (file_path, ALPHA, BETAS, LAMBDA_))
 
     result_thread_ndl, duration_parallel = clock(ndl.ndl,
                                                  (file_path, ALPHA, BETAS, LAMBDA_),
-                                                 number_of_threads=4, method='threading')
+                                                 n_jobs=4, method='threading')
 
     assert len(result_dict_ndl) == len(result_thread_ndl)
 
     unequal, unequal_ratio = compare_arrays(file_path, result_thread_ndl, result_dict_ndl)
     print('%.2f ratio unequal' % unequal_ratio)
-    assert len(unequal) == 0
+    assert len(unequal) == 0  # pylint: disable=len-as-condition
 
     print('parallel: %.3e  dict: %.3e' % (duration_parallel, duration_not_parallel))
     assert duration_parallel < duration_not_parallel
 
 
 def test_slice_list():
-    l1 = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
+    lst = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
 
-    res = ndl.slice_list(l1, 2)
+    res = ndl.slice_list(lst, 2)
     assert res == [[0, 1], [2, 3], [4, 5], [6, 7], [8, 9]]
 
-    res2 = ndl.slice_list(l1, 3)
+    res2 = ndl.slice_list(lst, 3)
     assert res2 == [[0, 1, 2], [3, 4, 5], [6, 7, 8], [9]]
 
 
-def clock(f, args, **kwargs):
+def clock(func, args, **kwargs):
     start = time.time()
-    result = f(*args, **kwargs)
+    result = func(*args, **kwargs)
     stop = time.time()
 
     duration = stop - start
@@ -429,11 +479,9 @@ def clock(f, args, **kwargs):
 
 
 def compare_arrays(file_path, arr1, arr2):
-    n_events, cues, outcomes = count.cues_outcomes(file_path)
-    cue_map, outcome_map, all_outcomes = generate_mapping(file_path)
+    _, cues, outcomes = count.cues_outcomes(file_path)
+    cue_map, outcome_map, _ = generate_mapping(file_path)
 
-    cue_indices = [cue_map[cue] for cue in cues]
-    outcome_indices = [outcome_map[outcome] for outcome in outcomes]
     unequal = list()
 
     for outcome in outcomes:
@@ -451,7 +499,7 @@ def compare_arrays(file_path, arr1, arr2):
                 else:
                     values.append(array[outcome][cue])
 
-            value1, value2 = values
+            value1, value2 = values  # pylint: disable=unbalanced-tuple-unpacking
             if not np.isclose(value1, value2, rtol=1e-02, atol=1e-05):
                 unequal.append((outcome, cue, value1, value2))
 
@@ -460,7 +508,7 @@ def compare_arrays(file_path, arr1, arr2):
 
 
 def generate_mapping(event_path):
-    n_events, cues, outcomes = count.cues_outcomes(event_path)
+    _, cues, outcomes = count.cues_outcomes(event_path)
     all_cues = list(cues.keys())
     all_outcomes = list(outcomes.keys())
     cue_map = OrderedDict(((cue, ii) for ii, cue in enumerate(all_cues)))
